!2345678911234567892123456789312345678941234567895123456789612345678971234567898
!        0         0         0         0         0         0         0
!
! Module: dpsssrc
!
! Author:  Th. Boller - XMM-SSC-MPE
!
! Description: Create copy of EPIC MAX-Likelihood detect list
!              create DPSS column ver_flag and ver_comm
!              create columns for source flags in Format 1A: A,E,N,P,G,X
!              set source flag not_checked (position 2) to T if
!                  DET_ML is below threshold
!
! Comments:
!
! $Id: dpsssrc_mod.f90,v 1.1.1.1 2003/10/23 09:42:57 eojero Exp $
!-
module dpsssrc_mod

  use dal
  use param
  use errorHandling

  implicit none

  private

  public dpsssrc

contains

  !---------------------------------------------------------------------
  subroutine dpsssrc
  !---------------------------------------------------------------------

  type(DataSetT)                                :: infile, outfile
  type(TableT)                                  :: outtab
  type(ColumnT)                                 :: ver_flag, ver_comm, det_ml
  type(ColumnT)					:: ind_sol
  type(ColumnT)					:: A
  type(ColumnT)					:: E
  type(ColumnT)					:: N
  type(ColumnT)					:: P
  type(ColumnT)					:: G
  type(ColumnT)					:: X

  real(single), dimension(:), pointer 		:: p_detml
  real 						:: thresh

  integer(kind=int32), dimension(:),  pointer	:: indsol

  character(len=1024)      			:: outname

  character(len=12)              		:: pnflag1
  character(len=1)              		:: pnflag2
  character(len=12)              		:: pnflag3
  character(len=40)              		:: comment
  character(len=31)              		:: xxx
  character(len=6)              		:: tttttt

  integer					   i, nr



 !pointer to input file
  infile = dataset(stringParameter("set"),MODIFY)
  xxx = stringParameter("set")


 !test if, M-L detect list is called TTTTTT = OMSRLI
  write(errstr,*) 'set = ',xxx
  call message(AppMsg,Verbose)

   !get string tttttt
    tttttt = xxx(18:23)
    write(errstr,*) "tttttt = ", tttttt
    call message(AppMsg,Verbose)

   !test for tttttt = EMSRLI
    if (tttttt .eq. "EMSRLI") then
        call message(AppMsg,Verbose," ML list found")
    else
        call error("badInput", "input PPS file is not the EPIC M-L detect list")
    endif




 !make a copy of the input file with prefix flag
!  write (outname,'(a,a)') "flag_",trim(stringParameter("set"))
   write (outname,'(a,a)') trim(stringParameter("prefix")),trim(stringParameter("set"))
  call copyDataset(stringParameter("set"),outname)


 !pointer to file and table
  outfile = dataSet(outname,MODIFY)
  outtab  = table(outfile,"SRCLIST",MODIFY)

 !create new columns (if not yet there)

  if (hasColumn(outtab,"IND_SOL")) then
  else
    ind_sol  = addColumn(outtab,"IND_SOL",INTEGER32,units="I", &
                         comment="running number")
  endif

  if (hasColumn(outtab,"VER_FLAG")) then
    ver_flag = column(outtab,"VER_FLAG",MODIFY)
  else
    ver_flag = addColumn(outtab,"VER_FLAG",String," ",(/12/))
  endif

  if (hasColumn(outtab,"A")) then
    A = column(outtab,"A",MODIFY)
  else
    A = addColumn(outtab,"A",String," ",(/1/))
  endif


  if (hasColumn(outtab,"E")) then
    E = column(outtab,"E",MODIFY)
  else
    E = addColumn(outtab,"E",String," ",(/1/))
  endif


  if (hasColumn(outtab,"N")) then
    N = column(outtab,"N",MODIFY)
  else
    N = addColumn(outtab,"N",String," ",(/1/))
  endif


  if (hasColumn(outtab,"P")) then
    P = column(outtab,"P",MODIFY)
  else
    P = addColumn(outtab,"P",String," ",(/1/))
  endif


  if (hasColumn(outtab,"G")) then
    G = column(outtab,"G",MODIFY)
  else
    G = addColumn(outtab,"G",String," ",(/1/))
  endif


  if (hasColumn(outtab,"X")) then
    X = column(outtab,"X",MODIFY)
  else
    X = addColumn(outtab,"X",String," ",(/1/))
  endif



  if (hasColumn(outtab,"VER_COMM")) then
    ver_comm = column(outtab,"VER_COMM",MODIFY)
  else
    ver_comm = addColumn(outtab,"VER_COMM",String," ",(/40/))
  endif

 !pointer to column ind_sol and to entries in column
  indsol => int32Data(ind_sol)

 !pointer to column ind_sol and to entries in column
  det_ml = column(outtab,"DET_ML",READ)
  p_detml => real32Data(det_ml)

  nr = numberOfRows(outtab)
  write(errstr,*) 'Number of rows: nr = ',nr
  call message(AppMsg,Verbose)

 !write column ind_sol
  do i = 0, nr - 1
     indsol(i) = i + 1
  enddo


 !write column ver_flag
  pnflag1 = "FFFFFFFFFFFF"
  do i = 0, nr - 1
     call setStringCell(ver_flag, i, pnflag1)
  enddo

 !write columns A
  pnflag2 = "A"
  do i = 0, nr - 1
     call setStringCell(A, i, pnflag2)
  enddo


!write columns E
  do i = 0, nr - 1
     call setStringCell(E, i, pnflag2)
  enddo


!write columns N
  do i = 0, nr - 1
     call setStringCell(N, i, pnflag2)
  enddo


!write columns P
  do i = 0, nr - 1
     call setStringCell(P, i, pnflag2)
  enddo


!write columns G
  do i = 0, nr - 1
     call setStringCell(G, i, pnflag2)
  enddo


!write columns X
  do i = 0, nr - 1
     call setStringCell(X, i, pnflag2)
  enddo


 !write column ver_comm
  comment = ".                                       "
  do i = 0, nr - 1
     call setStringCell(ver_comm, i, comment)
  enddo



 !make flag setting for not_checked sources
  thresh = realParameter("maxlikthresh")
  pnflag3 = "FTFFFFFFFFFF"
  do i = 0, nr - 1
    if (p_detml(i) < thresh) then
         call setStringCell(ver_flag, i, pnflag3)
    endif
  enddo


 !release columns
  call release (A)
  call release (E)
  call release (N)
  call release (P)
  call release (G)
  call release (X)

  call release (ind_sol)
  call release (ver_flag)
  call release (ver_comm)
  call release (det_ml)
  call release (outtab)
  call release (outfile)


  end subroutine dpsssrc
  !---------------------------------------------------------------------


end module dpsssrc_mod



